module Main where

import Conduit
import Options.Applicative.Simple
import Protolude

import qualified OpenAlex as OA
import qualified OpenAlex.Types as OA

main :: IO ()
main = do
  let filterHelp = help "Filter, for example: display_name.search:einstein , see https://docs.openalex.org/how-to-use-the-api/get-lists-of-entities/filter-entity-lists"
  (opts, runCmd) <-
    simpleOptions "0.1.0.0"
                  "OpenAlex"
                  "OpenAlex command line"
                  (pure ()) $ do
      addCommand "concepts"
        "Fetch OpenAlex concepts (https://docs.openalex.org/api-entities/concepts/concept-object)"
        fetchConcepts
        (strOption (long "filter"))
      addCommand "works"
        "Fetch OpenAlex works (https://docs.openalex.org/api-entities/works/work-object)"
        fetchWorks
        (strOption (long "filter"))
      addCommand "works-c"
        "Fetch OpenAlex works (https://docs.openalex.org/api-entities/works/work-object) with conduit"
        fetchWorksC
        (strOption (long "filter"))

  runCmd ()


fetchConcepts :: OA.Filter -> () -> IO ()
fetchConcepts fltr _ = do
  ec <- OA.fetchConcepts (Just 1) (Just 1) (Just "*") (Just fltr)
  case ec of
    Left err -> putText $ "error: " <> show err
    Right c -> do
      putText $ show c

fetchWorks :: OA.Filter -> () -> IO ()
fetchWorks fltr _ = do
  ew <- OA.fetchWorks (Just 1) (Just 1) (Just "*") (Just fltr)
  case ew of
    Left err -> putText $ "error: " <> show err
    Right w -> do
      putText $ show w

fetchWorksC :: OA.Filter -> () -> IO ()
fetchWorksC fltr _ = do
  eWorksC <- OA.fetchWorksC Nothing (Just fltr)
  case eWorksC of
    Left err -> putText $ "error: " <> show err
    Right (mCount, c) -> do
      putText $ "Count: " <> show mCount
      _ <- runConduit $ c
                       .| takeC 1000
                       .| mapM_C (\(OA.Work { .. }) -> do
                                     liftIO $ putText $ show id <> " :: " <> show display_name
                       )
      pure ()
